﻿Scriptname JaxonzEnhGrabDisplayCaseObject extends JaxonzEnhGrabObject  
{specialized version for 3 part display cases}

;import JaxonzUtility
;Bool bDebug = False

ObjectReference objActivator
ObjectReference objTrigger
ObjectReference objCase

int kSuperFast  = 100000

Bool Function Instantiate (ObjectReference objBase)
;initialize the object for positioning
;create references to associated objects
	if !flstValidForms.HasForm(objBase.GetBaseObject())
		return false	;not one of the object types appropriate for this script
	EndIf

	Int iIterations
	ObjectReference objRef
	Cell kCell = objBase.GetParentCell()

	;all parts share the same coordinates	
	;find parts
	iIterations = 100	;100
	While iIterations && !(objActivator && objTrigger && objCase)
		iIterations -= 1
		objRef = Game.FindRandomReferenceOfAnyTypeInListFromRef(flstValidForms, objBase, 3.0)	;almost exact same coordinates
		if (objRef as WeaponRackTriggerSCRIPT)
			objTrigger = objRef
		ElseIf (objRef as WeaponRackActivateSCRIPT)	;activate is invisible part
			objActivator = objRef
		Else	;case is the visible part
			objCase = objRef
		EndIf
	EndWhile
;TraceAndNotifyConditional("objTrigger: " + objTrigger, 0, bDebug)
;TraceAndNotifyConditional("objActivator: " + objActivator, 0, bDebug)
;TraceAndNotifyConditional("objCase: " + objCase, 0, bDebug)

	;double check that we have everything
	if !(objActivator && objTrigger && objCase)
		Debug.Notification("Can't find all parts of display case. Unsafe to move.")
		Release()
		return false
	Else
		;force reference to base object
		Self.ForceRefTo(objCase)
		return true
	EndIf	
EndFunction

Function TranslateTo(float afX, float afY, float afZ, float afXAngle, float afYAngle, float afZAngle, float afSpeed, float afMaxRotationSpeed = 0.0)
;same functionality as ObjectReference.TranslateTo except that coordinates associated items
	objCase.TranslateTo(afX, afY, afZ, afXAngle, afYAngle, afZAngle, afSpeed, afMaxRotationSpeed)
EndFunction
  
Function StopTranslation()
	objCase.StopTranslation()
	;nudge off perfect angle so that facing due north bug doesn't happen
	if ((objCase.GetAngleZ() as int) % 90) == 0
		objCase.TranslateTo(objCase.X, objCase.Y, objCase.Z, objCase.GetAngleX(), objCase.GetAngleY(), objCase.GetAngleZ() + 1.0, kSuperFast, 0.0)
	EndIf
	objActivator.TranslateToRef(objCase, kSuperFast, kSuperFast)
	;objActivator.MoveTo(objCase)
	objTrigger.TranslateToRef(objCase, kSuperFast, kSuperFast)
	;objTrigger.MoveTo(objCase)
EndFunction

Event OnTranslationComplete()
	StopTranslation()
EndEvent

ObjectReference[] Function Collect()
;hides/disables the objects, returns formlist of component objects
	ObjectReference[] objCollection = new ObjectReference[3]

	objCollection[0] = objCase
	objCollection[1] = objActivator
	objCollection[2] = objTrigger
	
	objCase.Disable()
	objActivator.Disable()
	objTrigger.Disable()
	;move objects with enableparents to limbo, since they can't be disabled via script.
	if objCase.IsEnabled()
		objCase.TranslateTo(objCase.X, objCase.Y, objCase.Z - 10000, 0,0,0,kSuperFast)
		objActivator.TranslateTo(objActivator.X, objActivator.Y, objActivator.Z - 10000, 0,0,0,kSuperFast)
		objTrigger.TranslateTo(objTrigger.X, objTrigger.Y, objTrigger.Z - 10000, 0,0,0,kSuperFast)
	EndIf

	Release()
	
	return objCollection
EndFunction

Function Restore (ObjectReference[] objCollection, ObjectReference objPlaceAtRef)
;unhides/enables objects at designated Location
	;assign references to appropriate parts
	;assumes array size and order
	objCase = objCollection[0]
	objActivator = objCollection[1]
	objTrigger = objCollection[2]
	
	;double check that we have everything
	if !(objActivator && objTrigger && objCase)
		Debug.Notification("Can't find all parts of display case. Unsafe to move.")
		Release()
		return
	Else
		;force reference to base object
		Self.ForceRefTo(objCase)
	EndIf	

;	;enable them
	objCase.Enable()
	objActivator.Enable()
	objTrigger.Enable()
;	;move to this cell
	;objCase.MoveTo(objPlaceAtRef)
	;objActivator.MoveTo(objPlaceAtRef)
	;objTrigger.MoveTo(objPlaceAtRef)
;	;rotate 180 degrees to it faces the player
	;objCase.SetAngle(0, 0, objPlaceAtRef.GetAngleZ() + 180.0)
	;objActivator.SetAngle(0, 0, objPlaceAtRef.GetAngleZ() + 180.0)
	;objTrigger.SetAngle(0, 0, objPlaceAtRef.GetAngleZ() + 180.0)
	TranslateTo(objPlaceAtRef.X, objPlaceAtRef.Y, objPlaceAtRef.Z, objPlaceAtRef.GetAngleX(), objPlaceAtRef.GetAngleY(), objPlaceAtRef.GetAngleZ() + 180.0, kSuperFast, 0.0)
EndFunction

Function Release()
;releases all internal references
	objCase = none
	objActivator = none
	objTrigger = none
	Self.Clear()
EndFunction

string Function GetDisplayName()
	return "Weapon Display Case"
EndFunction

float Function GetScale()
;returns current object scale
	return objCase.GetScale()
EndFunction

Function SetScale(float afScale)
;sets the object scale
	objCase.SetScale(afScale)
	objActivator.SetScale(afScale)
	objTrigger.SetScale(afScale)
EndFunction