﻿Scriptname JaxonzEnhGrabMannequinObject extends JaxonzEnhGrabObject  
{specialized version for mannequins}

ObjectReference objActivator
ObjectReference objMarker
ObjectReference objMannequin
Activator Property actMannequinActivateTrig Auto
ActorBase Property acbPlayerHouseMannequin Auto

int kActivator = 24
int kCharacter = 62
int kSuperFast  = 100000

;import JaxonzUtility
;bool bDebug = False

Bool Function Instantiate (ObjectReference objBase)
;initialize the object for positioning
;create references to associated objects
	if !flstValidForms.HasForm(objBase.GetBaseObject())
		return false	;not one of the object types appropriate for this script
	EndIf

	Int iIndex
	ObjectReference objRef
	Cell kCell = objBase.GetParentCell()
	
	if objBase.GetBaseObject() == actMannequinActivateTrig
		;find the mannequin
		objActivator = objBase
		iIndex = kCell.GetNumRefs(kCharacter)
		while (iIndex)
		    iIndex -= 1
		    objRef = kCell.GetNthRef(iIndex, kCharacter)
		    If objActivator.IsActivateChild(objRef)
				objMannequin = objRef
				;find the marker
				objMarker = objMannequin.GetLinkedRef()
				iIndex = 0
		    EndIf
		endwhile
	ElseIf objBase.GetBaseObject() == acbPlayerHouseMannequin
		;find the marker
		objMannequin = objBase
		objMarker = objMannequin.GetLinkedRef()
		;find the activator
		iIndex = kCell.GetNumRefs(kActivator)
		while (iIndex)
		    iIndex -= 1
		    objRef = kCell.GetNthRef(iIndex, kActivator)
		    If objRef.IsActivateChild(objMannequin)
				objActivator = objRef
				iIndex = 0
		    EndIf
		endwhile	
	EndIf

	;double check that we have everything for a mannequin
	if ! (objActivator && objMannequin && objMarker)
		Debug.Notification("Can't find all parts of mannequin. Unsafe to move.")
		Release()
		return false
	Else
		;force reference to base object
		Self.ForceRefTo(objMannequin)
		return true
	EndIf	
EndFunction

Function TranslateTo(float afX, float afY, float afZ, float afXAngle, float afYAngle, float afZAngle, float afSpeed, float afMaxRotationSpeed = 0.0)
;same functionality as ObjectReference.TranslateTo except that coordinates associated items
	objMannequin.TranslateTo(afX, afY, afZ, afXAngle, afYAngle, afZAngle, afSpeed, afMaxRotationSpeed)
	objMarker.TranslateTo(afX, afY, afZ, afXAngle, afYAngle, afZAngle, afSpeed, afMaxRotationSpeed)
EndFunction
  
Function StopTranslation()
	objMannequin.StopTranslation()
	objMarker.StopTranslation()
	objMarker.TranslateToRef(objMannequin, kSuperFast, 0.0)
	;objMarker.MoveTo(objMannequin)
	;nudge mannequin off perfect angle so that facing due north bug doesn't happen
	if ((objMarker.GetAngleZ() as int) % 90) == 0
		objMarker.TranslateTo(objMarker.X, objMarker.Y, objMarker.Z, objMarker.GetAngleX(), objMarker.GetAngleY(), objMarker.GetAngleZ() + 1.0, kSuperFast, 0.0)
		;objMarker.SetAngle(0, 0, objMarker.GetAngleZ() + 1)
	EndIf
	objActivator.TranslateTo(objMarker.X, objMarker.Y, objMarker.Z + 63.0, objMarker.GetAngleX(), objMarker.GetAngleY(), objMarker.GetAngleZ(), kSuperFast, 0.0)
	;objActivator.MoveTo(objMarker, 0, 0, 63.0)	
EndFunction

Event OnTranslationComplete()
	StopTranslation()
EndEvent

ObjectReference[] Function Collect()
;hides/disables the objects, returns reference to base object
	ObjectReference[] objCollection = new ObjectReference[3]
	objCollection[0] = objMannequin
	objCollection[1] = objActivator
	objCollection[2] = objMarker

	objMannequin.Disable()
	objActivator.Disable()
	objMarker.Disable()
	;move objects with enableparents to limbo, since they can't be disabled via script.
	if objMannequin.IsEnabled()
		;objMannequin.MoveTo((GetOwningQuest() as JaxonzEnhGrab).objLimboMarker)
		;objActivator.MoveTo((GetOwningQuest() as JaxonzEnhGrab).objLimboMarker)
		;objMarker.MoveTo((GetOwningQuest() as JaxonzEnhGrab).objLimboMarker)
		objMannequin.TranslateTo(objMannequin.X, objMannequin.Y, objMannequin.Z - 10000, 0,0,0,1000000)
		objActivator.TranslateTo(objActivator.X, objActivator.Y, objActivator.Z - 10000, 0,0,0,1000000)
		objMarker.TranslateTo(objMarker.X, objMarker.Y, objMarker.Z - 10000, 0,0,0,1000000)
	EndIf
	
	Release()
	
	return objCollection
EndFunction

Function Restore (ObjectReference[] objCollection, ObjectReference objPlaceAtRef)
;unhides/enables objects at designated Location
;TraceAndNotifyConditional("JaxonzEnhGrabMannequinObject Restore called", 0, bDebug)

	;assign references to appropriate parts
	;assumes array size and order
	objMannequin = objCollection[0]
	objActivator = objCollection[1]
	objMarker = objCollection[2]
	
	;double check that we have everything for a mannequin
	if ! (objActivator && objMannequin && objMarker)
		Debug.Notification("Can't find all parts of mannequin. Unsafe to move.")
		Release()
		return 
	Else
		;force reference to base object
		Self.ForceRefTo(objMannequin)	;possibly a CTD call?
	EndIf	

	;enable them
	objMannequin.Enable()
	objActivator.Enable()
	objMarker.Enable()
	
	;move to new location
	;objMannequin.MoveTo(objPlaceAtRef)
	;objActivator.MoveTo(objPlaceAtRef, 0, 0, 63.0)
	;objMarker.MoveTo(objPlaceAtRef)
	;rotate 180 degrees to it faces the player
	;objMannequin.SetAngle(0, 0, objPlaceAtRef.GetAngleZ() + 180.0)
	;objActivator.SetAngle(0, 0, objPlaceAtRef.GetAngleZ() + 180.0)
	;objMarker.SetAngle(0, 0, objPlaceAtRef.GetAngleZ() + 180.0)
	objMannequin.TranslateTo(objPlaceAtRef.X, objPlaceAtRef.Y, objPlaceAtRef.Z, objPlaceAtRef.GetAngleX(), objPlaceAtRef.GetAngleY(), objPlaceAtRef.GetAngleZ() + 180.0, kSuperFast, 0.0)
	objActivator.TranslateTo(objPlaceAtRef.X, objPlaceAtRef.Y, objPlaceAtRef.Z + 63.0, objPlaceAtRef.GetAngleX(), objPlaceAtRef.GetAngleY(), objPlaceAtRef.GetAngleZ() + 180.0, kSuperFast, 0.0)
	objMarker.TranslateTo(objPlaceAtRef.X, objPlaceAtRef.Y, objPlaceAtRef.Z, objPlaceAtRef.GetAngleX(), objPlaceAtRef.GetAngleY(), objPlaceAtRef.GetAngleZ() + 180.0, kSuperFast, 0.0)
	
	;make it visible by activating the trigger
	objActivator.Activate(Game.GetPlayer())
	;wait for menu to open
	While !Utility.IsInMenuMode()
;		Utility.WaitMenuMode(0.1)
	EndWhile
	;wait for menu to close
	While Utility.IsInMenuMode()
;		Utility.WaitMenuMode(0.1)
		;close the menu
		Input.TapKey(Input.GetMappedKey("Tween Menu"))
	EndWhile
	
	;have mannequin put clothes back on
	int iItemCount = objMannequin.GetNumItems()
	While iItemCount
		iItemCount -= 1
		(objMannequin as Actor).EquipItem(objMannequin.GetNthForm(iItemCount))
	EndWhile
EndFunction

Function Release()
;releases all internal references
	objMannequin = none
	objActivator = none
	objMarker = none
	Self.Clear()
EndFunction

string Function GetDisplayName()
	return "Mannequin"
EndFunction

float Function GetScale()
;returns current object scale
	return objMannequin.GetScale()
EndFunction

Function SetScale(float afScale)
;sets the object scale
	objMannequin.SetScale(afScale)
	objActivator.SetScale(afScale)
	objMarker.SetScale(afScale)
EndFunction