﻿Scriptname JaxonzEnhGrabWeaponPlaqueObject extends JaxonzEnhGrabObject  
{specialized version for 2 part weapon racks, including plaques and wooden weapon racks}

;import JaxonzUtility
;Bool bDebug = False

ObjectReference objActivator
ObjectReference objPlaque

int kSuperFast  = 100000

Bool Function Instantiate (ObjectReference objBase)
;initialize the object for positioning
	if !flstValidForms.HasForm(objBase.GetBaseObject())
		return false	;not one of the object types appropriate for this script
	EndIf


	Int iIterations
	ObjectReference objRef
	Cell kCell = objBase.GetParentCell()
	
	;find weapon plaque parts
	if (objBase as WeaponRackActivateSCRIPT)	;activate is invisible part
		objActivator = objBase
;TraceAndNotifyConditional("objActivator: " + objActivator, 0, bDebug)
		iIterations = 100
		While iIterations && !objPlaque
			objRef = Game.FindRandomReferenceOfAnyTypeInListFromRef(flstValidForms, objBase, 5.0)	;almost exact same coordinates
			if (objRef as WeaponRackTriggerSCRIPT)
				objPlaque = objRef
;TraceAndNotifyConditional("objPlaque: " + objPlaque, 0, bDebug)
			EndIf
		EndWhile
	ElseIf (objBase as WeaponRackTriggerSCRIPT)	;trigger is visible part
		objPlaque =  objBase
;TraceAndNotifyConditional("objPlaque: " + objPlaque, 0, bDebug)
		iIterations = 100
		While iIterations && !objActivator
			objRef = Game.FindRandomReferenceOfAnyTypeInListFromRef(flstValidForms, objBase, 5.0)	;almost exact same coordinates
			if (objRef as WeaponRackActivateSCRIPT)
				objActivator = objRef
;TraceAndNotifyConditional("objActivator: " + objActivator, 0, bDebug)
			EndIf
		EndWhile
	Else
;TraceAndNotifyConditional("objBase couldnt be cast as WeaponRackActivateSCRIPT or WeaponRackTriggerSCRIPT")
	EndIf

;TraceAndNotifyConditional("objActivator: " + objActivator, 0, bDebug)
;TraceAndNotifyConditional("objPlaque: " + objPlaque, 0, bDebug)

	;double check that we have everything
	if ! (objActivator && objPlaque)
		Debug.Notification("Can't find all parts of weapon plaque. Unsafe to move.")
		Release()
		return false
	Else
		;force reference to base object
		Self.ForceRefTo(objPlaque)
		return true
	EndIf	
EndFunction

Function TranslateTo(float afX, float afY, float afZ, float afXAngle, float afYAngle, float afZAngle, float afSpeed, float afMaxRotationSpeed = 0.0)
;same functionality as ObjectReference.TranslateTo except that coordinates associated items
;	objPlaque.StopTranslation()	;for safety, ALWAYS call StopTranslation before TranslateTo
	objPlaque.TranslateTo(afX, afY, afZ, afXAngle, afYAngle, afZAngle, afSpeed, afMaxRotationSpeed)
EndFunction
  
Function StopTranslation()
	if objPlaque && objActivator
;TraceAndNotifyConditional("JaxonzEnhGrabWeaponPlaqueObject StopTranslation", 0, bDebug)
		objPlaque.StopTranslation()
		;nudge off perfect angle so that facing due north bug doesn't happen
		if ((objPlaque.GetAngleZ() as int) % 90) == 0
			objPlaque.TranslateTo(objPlaque.X, objPlaque.Y, objPlaque.Z, objPlaque.GetAngleX(), objPlaque.GetAngleY(), objPlaque.GetAngleZ() + 1.0, kSuperFast, 0.0)
			;objPlaque.SetAngle(0, 0, objPlaque.GetAngleZ() + 1)
		EndIf
		objActivator.TranslateToRef(objPlaque, kSuperFast, 0.0)
		;objActivator.MoveTo(objPlaque)
	EndIf
;TraceAndNotifyConditional("JaxonzEnhGrabWeaponPlaqueObject StopTranslation exit", 0, bDebug)
EndFunction

Event OnTranslationComplete()
	StopTranslation()
EndEvent

ObjectReference[] Function Collect()
;hides/disables the objects, returns reference to base object
;TraceAndNotifyConditional("JaxonzEnhGrabWeaponPlaqueObject Collect called", 0, bDebug)

	ObjectReference[] objCollection = new ObjectReference[2]
	objCollection[0] = objPlaque
	objCollection[1] = objActivator
	objPlaque.Disable()
	objActivator.Disable()
	;move objects with enableparents to limbo, since they can't be disabled via script.
	if objPlaque.IsEnabled()
;		objPlaque.MoveTo((GetOwningQuest() as JaxonzEnhGrab).objLimboMarker)
;		objActivator.MoveTo((GetOwningQuest() as JaxonzEnhGrab).objLimboMarker)
		objPlaque.TranslateTo(objPlaque.X, objPlaque.Y, objPlaque.Z - 10000, 0,0,0,1000000)
		objActivator.TranslateTo(objActivator.X, objActivator.Y, objActivator.Z - 10000, 0,0,0,1000000)
	EndIf

	Release()
	
;TraceAndNotifyConditional("JaxonzEnhGrabWeaponPlaqueObject Collect objCollection: " + objCollection as string, 0, bDebug)

	return objCollection
EndFunction

Function Restore (ObjectReference[] objCollection, ObjectReference objPlaceAtRef)
;unhides/enables objects at designated Location
	;assign ObjectReferences to appropriate variables
	;assumes array size and order
	objPlaque = objCollection[0]
	objActivator = objCollection[1]
	
	;double check that we have everything
	if ! (objActivator && objPlaque)
		Debug.Notification("Can't find all parts of weapon plaque. Unsafe to move.")
		Release()
		return
	Else
		;force reference to base object
		Self.ForceRefTo(objPlaque)
	EndIf	

	;enable them
	objPlaque.Enable()
	objActivator.Enable()
	;move to this cell
	;objPlaque.MoveTo(objPlaceAtRef)
	;objActivator.MoveTo(objPlaceAtRef)
	;rotate 180 degrees to it faces the player
	;objPlaque.SetAngle(0, 0, objPlaceAtRef.GetAngleZ() + 180.0)
	;objActivator.SetAngle(0, 0, objPlaceAtRef.GetAngleZ() + 180.0)
	objPlaque.TranslateTo(objPlaceAtRef.X, objPlaceAtRef.Y, objPlaceAtRef.Z, objPlaceAtRef.GetAngleX(), objPlaceAtRef.GetAngleY(), objPlaceAtRef.GetAngleZ() + 180.0, kSuperFast, 0.0)
	objActivator.TranslateToRef(objPlaque, kSuperFast, 0.0)
	
EndFunction

Function Release()
;releases all internal references
	objPlaque = none
	objActivator = none
	Self.Clear()
EndFunction

string Function GetDisplayName()
	return "Weapon Plaque"
EndFunction

float Function GetScale()
;returns current object scale
	return objPlaque.GetScale()
EndFunction

Function SetScale(float afScale)
;sets the object scale
	objActivator.SetScale(afScale)
	objPlaque.SetScale(afScale)
EndFunction