﻿Scriptname JaxonzSKSEVersionCheckScript extends Quest

import Game
import Debug
import SKSE

String Property strModName = "Jaxonz" Auto	;this value should always be populated
String Property sSKSEVersionRequired = "1.7.1" Auto
Int Property iMinVersionRelease = 46 Auto  
int Property iWarnUser = 1 Auto  
	int iWarnNever = 0 
	int iWarnOnFail = 1 
	int iWarnAlways = 2
int Property iSkyUIRequired = 1 Auto
	int iSkyUINotRequired = 0 
	int iSkyUINotOptional = 1 
	int iSkyUIRequired = 2
string Property sUserWarningMessage Auto
string[] Property sRequiredESPs Auto

bool bSKSEsuccess
bool bSKSEscriptsuccess
bool bSkyUIsuccess
bool bESPsuccess

Event OnInit()	;first time script initiated
	Trace("JaxonzSKSEVersionCheckScript OnInit")
	if IsRunning()
		CheckRequirements(iWarnUser)
	EndIf
EndEvent

bool Function CheckRequirements(int iWarnUserFlag)	;perform all checks, can be called by external routines with iWarnAlways to explicitly check requirements and notify user
	bSKSEsuccess = CheckSKSEVersion(iWarnUserFlag)
	;bSKSEscriptsuccess = TestSKSEScripts(iWarnUserFlag)
	bSkyUIsuccess = CheckSkyUI(iWarnUserFlag)
	bESPsuccess = CheckRequiredESPs(iWarnUserFlag)
	return bSKSEsuccess && bSkyUIsuccess && bESPsuccess ;&& bSKSEscriptsuccess
EndFunction

Bool Function CheckSKSEVersion(int iWarnUserFlag)
;check the current SKSE version and display a message if below minimum required
	Int iVersionRelease
	String CurrentSKSEVersionString
	string strWarning = "SKSE check:\n\n"
	String strSKSEURL = "\nPlease visit http://skse.silverlock.org/ for the newest version."
	
	iVersionRelease = GetVersionRelease()
	
	CurrentSKSEVersionString =  GetVersion() + "." + GetVersionMinor() + "." + GetVersionBeta() + " rel " + iVersionRelease + " installed"
	
	if iVersionRelease
		strWarning += CurrentSKSEVersionString
		If (iVersionRelease < iMinVersionRelease)
			strWarning = "\n" + strModName  + " was compiled with SKSE rel " + sSKSEVersionRequired + ".\n\nThe SKSE version installed may not be sufficient." + strSKSEURL
		Else
			strWarning += "\n\nSuccess!"
		EndIf
	Else
		strWarning += "SKSE is not installed or Skyrim was not started via SKSE_loader.exe with sufficient privilege.\n" + \
			strModName + " will not run correctly." + strSKSEURL
	EndIf
	
	Trace(strWarning + "\n(rel " + iMinVersionRelease + " required)")
	
	;notify user
	if iWarnUserFlag == iWarnNever
		;supress warning messages
	ElseIf iWarnUserFlag == iWarnAlways
		MessageBox(strWarning)	
	ElseIf iVersionRelease < iMinVersionRelease	;warn on fail
		MessageBox(strWarning)
	endif
	
	Return iVersionRelease >= iMinVersionRelease
EndFunction

Bool Function CheckSkyUI(int iWarnUserFlag)
;check if SkyUI version and display a message if below minimum required)
	string strWarning = "SkyUI check:\n\n"
	
	bool bSkyUIInstalled = GetFormFromFile(0x0000802, "SkyUI.esp")	;SKI_ConfigManagerInstance quest form
	if bSkyUIInstalled
		strWarning += "SkyUI: Installed\n\nSuccess!"
	Else
		strWarning += "SkyUI: MISSING\n\n"
		if iSkyUIRequired == iSkyUINotOptional
			strWarning += "Some features may not be available."
		ElseIf (iSkyUIRequired == iSkyUIRequired)
			strWarning += strModName + " will not run correctly without it. Please install SkyUI."
		EndIf
	EndIf

	Trace(strWarning)

	;notify user
	if iWarnUserFlag == iWarnNever
		;do nothing
	ElseIf iWarnUserFlag == iWarnAlways
		MessageBox(strWarning)
	ElseIf !bSkyUIInstalled
		MessageBox(strWarning)
	EndIf

	return bSkyUIInstalled
EndFunction

Bool Function CheckRequiredESPs(int iWarnUserFlag)
	int iIndex = 0
	bool bMissingMod = false
	string strWarning = "Required Mods:\n"
	

	if sRequiredESPs.Length == 0
		strWarning += "[none]\n"
	EndIf

	While iIndex < sRequiredESPs.Length
		strWarning += sRequiredESPs[iIndex] + ": "
		If GetModByName(sRequiredESPs[iIndex]) == 255	;mod not loaded. NOTE THAT THIS IS AN SKSE FUNCTION THAT COULD PRODUCE FALSE POSITIVE IF SKSE NOT INSTALLED

;Altnerative non-SKSE based check isn't working
;		form frmBase
;		frmBase = GetFormFromFile(0x00000000, sRequiredESPs[iIndex])	;base form of each ESP
;Trace(sRequiredESPs[iIndex] + " frmBase: " + frmBase)
;		if !frmBase	
			bMissingMod = true
			strWarning += "MISSING\n"
		Else
			strWarning += "installed\n"
		EndIf
		iIndex += 1
	EndWhile

	if bMissingMod
		strWarning += "\nFAILED! May not run correctly."
	Else
		strWarning += "\nSuccess!"
	EndIf
	
	if !bSKSEsuccess
		strWarning += "\n(Bad SKSE could result in false failures.)"
	endif

	Trace(strWarning)

	;notify user
	if iWarnUserFlag == iWarnNever
		;do nothing, supress warning
	ElseIf (iWarnUserFlag == iWarnAlways) || bMissingMod
		MessageBox(strWarning)
	EndIf

	return !bMissingMod
EndFunction

Bool Function TestSKSEScripts(int iWarnUserFlag)
;exercise script functions to validate SKSE installation
	string strWarning = strModName + " SKSE script test:\n\n"
	bool bTestSuccess
	
	if iMinVersionRelease >= 46
		Form frmIronDagger = GetForm(0x0001397E)
Trace("frmIronDagger:" + frmIronDagger)
		bTestSuccess = frmIronDagger && frmIronDagger.IsPlayable()
	Else
		bTestSuccess = true
	EndIf
	
	if bTestSuccess
		strWarning += "Success!"
	Else
		strWarning += "FAILURE: Unable to call SKSE " + sSKSEVersionRequired + " functions.\nIt is likely that SKSE script files (*.pex) are not correctly installed."
	EndIf
	
	;notify user
	if iWarnUserFlag == iWarnNever
		;do nothing, supress warning
	ElseIf (iWarnUserFlag == iWarnAlways) || !bTestSuccess
		MessageBox(strWarning)
	EndIf
	
EndFunction

Function RunSKSETestBattery()
	;run a test on common SKSE functions and show results report
	Actor akPlayer = GetPlayer() as Actor
	
	String strResults = "Results of Exercising Common SKSE Functions:\n"
	
	strResults +=  "\nStringUtil.Find(): " 
	if StringUtil.Find("Zero if broken", "if")
		strResults += "PASS"
	Else
		strResults += "FAIL"
	EndIf
			
;	strResults = strResults + "akPlayer.GetHairColor(): " + akPlayer.GetHairColor() + "\n"
	strResults += "\nStringUtil.GetLength(): "
	if StringUtil.GetLength("four") == 4
		strResults += "PASS"
	Else
		strResults += "FAIL"
	EndIf
			
	strResults += "\nForm.GetName(): "
	if (Self as Quest).GetName()
		strResults += "PASS"
	Else
		strResults += "FAIL"
	EndIf
			
	strResults += "\nPlayer.GetNthForm(0): "  
	if akPlayer.GetNthForm(0)
		strResults += "PASS"
	Else
		strResults += "FAIL"
	EndIf
			
	strResults += "\nActor.GetSpellCount(): " 
	if akPlayer.GetSpellCount()
		strResults += "PASS"
	Else
		strResults += "FAIL"
	EndIf
			
;	strResults += "akPlayer.GetNthSpell(0).GetName(): " + akPlayer.GetNthSpell(0).GetName() + "\n"
	strResults += "\nForm.GetType(): " 
	if akPlayer.GetType()
		strResults += "PASS"
	Else
		strResults += "FAIL"
	EndIf
			
	strResults += "\nForm.IsPlayable(): " 
	if GetForm(0x0001397E).IsPlayable()
		strResults += "PASS"
	Else
		strResults += "FAIL, SKSE 1.7.1 likely not installed"
	EndIf
			
;	strResults += "\nCell.GetWaterLevel(): " 
;	if akPlayer.GetParentCell().GetWaterLevel()
;		strResults += "PASS"
;	Else
;		strResults += "FAIL, SKSE 1.7.2 likely not installed"
;	EndIf

;	strResults += "akPlayer.GetWeight(): " + akPlayer.GetActorBase().GetWeight() + "\n"
;	strResults += ": " +  + "\n"
;	strResults += "\nZero, false or empty results indicate an error with SKSE."

	MessageBox(strResults)
EndFunction